#include <stdio.h>					// puts
#include <stdlib.h>					// atoi
#include <windows.h>
#include <iphlpapi.h>

const char* CheckOption(const char* check, const char* match);

int main(int argc, char *argv[])
{
	bool usage = false;

	int arg = 1;
	for (arg = 1; !usage && arg < argc; arg++) {
		if (argv[arg][0] != '-')
			break;

		if (CheckOption(argv[arg], "-help"))
			usage = true;
		else
			printf("Unknown option: %s\n", argv[arg]);
	}

	if (usage) {
		puts("usage: wine show-eths.exe.so\n");
		puts("Use Windows ethernet interface discovery APIs to print information");
		puts("about the available interfaces.");
		puts("OPTIONS:");
		puts("	-help    prints this useful info");
		
		return 0;
	}

	PIP_ADAPTER_INFO pAdapterInfo;
	PIP_ADAPTER_INFO pAdapter = NULL;
	DWORD dwRetVal = 0;
	
	pAdapterInfo = (IP_ADAPTER_INFO *) malloc( sizeof(IP_ADAPTER_INFO) );
	ULONG ulOutBufLen = sizeof(IP_ADAPTER_INFO);
	
	// Make an initial call to GetAdaptersInfo to get
	// the necessary size into the ulOutBufLen variable
	if (GetAdaptersInfo( pAdapterInfo, &ulOutBufLen) == ERROR_BUFFER_OVERFLOW) {
		free(pAdapterInfo);
		pAdapterInfo = (IP_ADAPTER_INFO *) malloc (ulOutBufLen); 
	}
	
	if ((dwRetVal = GetAdaptersInfo( pAdapterInfo, &ulOutBufLen)) == NO_ERROR) {
		pAdapter = pAdapterInfo;
		while (pAdapter) {
			printf("\tAdapter Name: \t%s\n", pAdapter->AdapterName);
			printf("\tAdapter Desc: \t%s\n", pAdapter->Description);
			printf("\tAdapter Addr: \t%.02X:%.02X:%.02X:%.02X:%.02X:%.02X\n", pAdapter->Address[0],
					pAdapter->Address[1], pAdapter->Address[2], pAdapter->Address[3],
					pAdapter->Address[4], pAdapter->Address[5]);
			printf("\tIP Address: \t%s\n", pAdapter->IpAddressList.IpAddress.String);
			printf("\tIP Mask: \t%s\n", pAdapter->IpAddressList.IpMask.String);
	
			printf("\tGateway: \t%s\n", pAdapter->GatewayList.IpAddress.String);
			printf("\t***\n");
			if (pAdapter->DhcpEnabled) {
				printf("\tDHCP Enabled: Yes\n");
				printf("\t\tDHCP Server: \t%s\n", pAdapter->DhcpServer.IpAddress.String);
				printf("\tLease Obtained: %ld\n", pAdapter->LeaseObtained);
			}
			else
				printf("\tDHCP Enabled: No\n");
			
			if (pAdapter->HaveWins) {
				printf("\tHave Wins: Yes\n");
				printf("\t\tPrimary Wins Server: \t%s\n", pAdapter->PrimaryWinsServer.IpAddress.String);
				printf("\t\tSecondary Wins Server: \t%s\n", pAdapter->SecondaryWinsServer.IpAddress.String);
			}
			else
				printf("\tHave Wins: No\n");
			pAdapter = pAdapter->Next;
		}
	}
	else {
		printf("Call to GetAdaptersInfo failed.\n");
	}

	puts("\n~FIN~");
	return 0;
}

const char* CheckOption(const char* check, const char* match)
{
	size_t length = strlen(match);
	if (match[length - 1] != '=') {
		if (strcmp(check, match) == 0)
			return "";
	}
	else if (strlen(check) > length) {
		if (memcmp(check, match, length) == 0)
			return check + length;
	}

	return 0;
}

